@@ -1,11 +1,11 @@
 /*
  * kmp_csupport.c -- kfront linkage support for OpenMP.
- * $Revision: 42826 $
- * $Date: 2013-11-20 03:39:45 -0600 (Wed, 20 Nov 2013) $
+ * $Revision: 43473 $
+ * $Date: 2014-09-26 15:02:57 -0500 (Fri, 26 Sep 2014) $
  */
 
 /* <copyright>
-    Copyright (c) 1997-2013 Intel Corporation.  All Rights Reserved.
+    Copyright (c) 1997-2014 Intel Corporation.  All Rights Reserved.
 
     Redistribution and use in source and binary forms, with or without
     modification, are permitted provided that the following conditions
@@ -39,6 +39,7 @@
 #include "kmp_i18n.h"
 #include "kmp_itt.h"
 #include "kmp_error.h"
+#include "kmp_stats.h"
 
 #define MAX_MESSAGE 512
 
@@ -54,7 +55,7 @@
  * @param flags in   for future use (currently ignored)
  *
  * Initialize the runtime library. This call is optional; if it is not made then
- * it will be implicilty called by attempts to use other library functions.
+ * it will be implicitly called by attempts to use other library functions.
  *
  */
 void
@@ -295,13 +296,18 @@ Do the actual fork and call the microtask in the relevant number of threads.
 void
 __kmpc_fork_call(ident_t *loc, kmp_int32 argc, kmpc_micro microtask, ...)
 {
+  KMP_STOP_EXPLICIT_TIMER(OMP_serial);
+  KMP_COUNT_BLOCK(OMP_PARALLEL);
   int         gtid = __kmp_entry_gtid();
   // maybe to save thr_state is enough here
   {
     va_list     ap;
     va_start(   ap, microtask );
 
-    __kmp_fork_call( loc, gtid, TRUE,
+#if INCLUDE_SSC_MARKS
+    SSC_MARK_FORKING();
+#endif
+    __kmp_fork_call( loc, gtid, fork_context_intel,
             argc,
             VOLATILE_CAST(microtask_t) microtask,
             VOLATILE_CAST(launch_t)    __kmp_invoke_task_func,
@@ -312,10 +318,14 @@ __kmpc_fork_call(ident_t *loc, kmp_int32 argc, kmpc_micro microtask, ...)
             ap
 #endif
             );
+#if INCLUDE_SSC_MARKS
+    SSC_MARK_JOINING();
+#endif
     __kmp_join_call( loc, gtid );
 
     va_end( ap );
   }
+  KMP_START_EXPLICIT_TIMER(OMP_serial);
 }
 
 #if OMP_40_ENABLED
@@ -356,17 +366,18 @@ __kmpc_fork_teams(ident_t *loc, kmp_int32 argc, kmpc_micro microtask, ...)
     va_start(   ap, microtask );
 
     // remember teams entry point and nesting level
-    this_thr->th.th_team_microtask = microtask;
+    this_thr->th.th_teams_microtask = microtask;
     this_thr->th.th_teams_level = this_thr->th.th_team->t.t_level; // AC: can be >0 on host
 
     // check if __kmpc_push_num_teams called, set default number of teams otherwise
-    if ( this_thr->th.th_set_nth_teams == 0 ) {
+    if ( this_thr->th.th_teams_size.nteams == 0 ) {
         __kmp_push_num_teams( loc, gtid, 0, 0 );
     }
     KMP_DEBUG_ASSERT(this_thr->th.th_set_nproc >= 1);
-    KMP_DEBUG_ASSERT(this_thr->th.th_set_nth_teams >= 1);
+    KMP_DEBUG_ASSERT(this_thr->th.th_teams_size.nteams >= 1);
+    KMP_DEBUG_ASSERT(this_thr->th.th_teams_size.nth >= 1);
 
-    __kmp_fork_call( loc, gtid, TRUE,
+    __kmp_fork_call( loc, gtid, fork_context_intel,
             argc,
             VOLATILE_CAST(microtask_t) __kmp_teams_master,
             VOLATILE_CAST(launch_t)    __kmp_invoke_teams_master,
@@ -377,9 +388,9 @@ __kmpc_fork_teams(ident_t *loc, kmp_int32 argc, kmpc_micro microtask, ...)
 #endif
             );
     __kmp_join_call( loc, gtid );
-    this_thr->th.th_team_microtask = NULL;
+    this_thr->th.th_teams_microtask = NULL;
     this_thr->th.th_teams_level = 0;
-
+    *(kmp_int64*)(&this_thr->th.th_teams_size) = 0L;
     va_end( ap );
 }
 #endif /* OMP_40_ENABLED */
@@ -412,252 +423,9 @@ when the condition is false.
 void
 __kmpc_serialized_parallel(ident_t *loc, kmp_int32 global_tid)
 {
-    kmp_info_t *this_thr;
-    kmp_team_t *serial_team;
-
-    KC_TRACE( 10, ("__kmpc_serialized_parallel: called by T#%d\n", global_tid ) );
-
-    /* Skip all this code for autopar serialized loops since it results in
-       unacceptable overhead */
-    if( loc != NULL && (loc->flags & KMP_IDENT_AUTOPAR ) )
-        return;
-
-    if( ! TCR_4( __kmp_init_parallel ) )
-        __kmp_parallel_initialize();
-
-    this_thr     = __kmp_threads[ global_tid ];
-    serial_team  = this_thr -> th.th_serial_team;
-
-    /* utilize the serialized team held by this thread */
-    KMP_DEBUG_ASSERT( serial_team );
-    KMP_MB();
-
-#if OMP_30_ENABLED
-    if ( __kmp_tasking_mode != tskm_immediate_exec ) {
-        KMP_DEBUG_ASSERT( this_thr -> th.th_task_team == this_thr -> th.th_team -> t.t_task_team );
-        KMP_DEBUG_ASSERT( serial_team -> t.t_task_team == NULL );
-        KA_TRACE( 20, ( "__kmpc_serialized_parallel: T#%d pushing task_team %p / team %p, new task_team = NULL\n",
-                        global_tid, this_thr -> th.th_task_team, this_thr -> th.th_team ) );
-        this_thr -> th.th_task_team = NULL;
-    }
-#endif // OMP_30_ENABLED
-
-#if OMP_40_ENABLED
-    kmp_proc_bind_t proc_bind = this_thr->th.th_set_proc_bind;
-    if ( this_thr->th.th_current_task->td_icvs.proc_bind == proc_bind_false ) {
-        proc_bind = proc_bind_false;
-    }
-    else if ( proc_bind == proc_bind_default ) {
-        //
-        // No proc_bind clause was specified, so use the current value
-        // of proc-bind-var for this parallel region.
-        //
-        proc_bind = this_thr->th.th_current_task->td_icvs.proc_bind;
-    }
-    //
-    // Reset for next parallel region
-    //
-    this_thr->th.th_set_proc_bind = proc_bind_default;
-#endif /* OMP_3_ENABLED */
-
-    if( this_thr -> th.th_team != serial_team ) {
-#if OMP_30_ENABLED
-        // Nested level will be an index in the nested nthreads array
-        int level = this_thr->th.th_team->t.t_level;
-#endif
-        if( serial_team -> t.t_serialized ) {
-            /* this serial team was already used
-             * TODO increase performance by making this locks more specific */
-            kmp_team_t *new_team;
-            int tid = this_thr->th.th_info.ds.ds_tid;
-
-            __kmp_acquire_bootstrap_lock( &__kmp_forkjoin_lock );
-
-            new_team = __kmp_allocate_team(this_thr->th.th_root, 1, 1,
-#if OMP_40_ENABLED
-                                           proc_bind,
-#endif
-#if OMP_30_ENABLED
-                                           & this_thr->th.th_current_task->td_icvs,
-#else
-                                           this_thr->th.th_team->t.t_set_nproc[tid],
-                                           this_thr->th.th_team->t.t_set_dynamic[tid],
-                                           this_thr->th.th_team->t.t_set_nested[tid],
-                                           this_thr->th.th_team->t.t_set_blocktime[tid],
-                                           this_thr->th.th_team->t.t_set_bt_intervals[tid],
-                                           this_thr->th.th_team->t.t_set_bt_set[tid],
-#endif // OMP_30_ENABLED
-                                           0);
-            __kmp_release_bootstrap_lock( &__kmp_forkjoin_lock );
-            KMP_ASSERT( new_team );
-
-            /* setup new serialized team and install it */
-            new_team -> t.t_threads[0] = this_thr;
-            new_team -> t.t_parent = this_thr -> th.th_team;
-            serial_team = new_team;
-            this_thr -> th.th_serial_team = serial_team;
-
-            KF_TRACE( 10, ( "__kmpc_serialized_parallel: T#%d allocated new serial team %p\n",
-                            global_tid, serial_team ) );
-
-
-            /* TODO the above breaks the requirement that if we run out of
-             * resources, then we can still guarantee that serialized teams
-             * are ok, since we may need to allocate a new one */
-        } else {
-            KF_TRACE( 10, ( "__kmpc_serialized_parallel: T#%d reusing cached serial team %p\n",
-                            global_tid, serial_team ) );
-        }
-
-        /* we have to initialize this serial team */
-        KMP_DEBUG_ASSERT( serial_team->t.t_threads );
-        KMP_DEBUG_ASSERT( serial_team->t.t_threads[0] == this_thr );
-        KMP_DEBUG_ASSERT( this_thr->th.th_team != serial_team );
-        serial_team -> t.t_ident         = loc;
-        serial_team -> t.t_serialized    = 1;
-        serial_team -> t.t_nproc         = 1;
-        serial_team -> t.t_parent        = this_thr->th.th_team;
-#if OMP_30_ENABLED
-        serial_team -> t.t_sched         = this_thr->th.th_team->t.t_sched;
-#endif // OMP_30_ENABLED
-        this_thr -> th.th_team           = serial_team;
-        serial_team -> t.t_master_tid    = this_thr->th.th_info.ds.ds_tid;
-
-#if OMP_30_ENABLED
-        KF_TRACE( 10, ( "__kmpc_serialized_parallel: T#d curtask=%p\n",
-                        global_tid, this_thr->th.th_current_task ) );
-        KMP_ASSERT( this_thr->th.th_current_task->td_flags.executing == 1 );
-        this_thr->th.th_current_task->td_flags.executing = 0;
-
-        __kmp_push_current_task_to_thread( this_thr, serial_team, 0 );
-
-        /* TODO: GEH: do the ICVs work for nested serialized teams?  Don't we need an implicit task for
-           each serialized task represented by team->t.t_serialized? */
-        copy_icvs(
-                  & this_thr->th.th_current_task->td_icvs,
-                  & this_thr->th.th_current_task->td_parent->td_icvs );
-
-        // Thread value exists in the nested nthreads array for the next nested level
-        if ( __kmp_nested_nth.used && ( level + 1 < __kmp_nested_nth.used ) ) {
-            this_thr->th.th_current_task->td_icvs.nproc = __kmp_nested_nth.nth[ level + 1 ];
-        }
-
-#if OMP_40_ENABLED
-        if ( __kmp_nested_proc_bind.used && ( level + 1 < __kmp_nested_proc_bind.used ) ) {
-            this_thr->th.th_current_task->td_icvs.proc_bind
-                = __kmp_nested_proc_bind.bind_types[ level + 1 ];
-        }
-#endif /* OMP_40_ENABLED */
-
-#else /* pre-3.0 icv's */
-        serial_team -> t.t_set_nproc[0]  = serial_team->t.t_parent->
-            t.t_set_nproc[serial_team->
-                          t.t_master_tid];
-        serial_team -> t.t_set_dynamic[0] = serial_team->t.t_parent->
-            t.t_set_dynamic[serial_team->
-                            t.t_master_tid];
-        serial_team -> t.t_set_nested[0] = serial_team->t.t_parent->
-            t.t_set_nested[serial_team->
-                           t.t_master_tid];
-        serial_team -> t.t_set_blocktime[0]  = serial_team->t.t_parent->
-            t.t_set_blocktime[serial_team->
-                              t.t_master_tid];
-        serial_team -> t.t_set_bt_intervals[0] = serial_team->t.t_parent->
-            t.t_set_bt_intervals[serial_team->
-                                 t.t_master_tid];
-        serial_team -> t.t_set_bt_set[0] = serial_team->t.t_parent->
-            t.t_set_bt_set[serial_team->
-                           t.t_master_tid];
-#endif // OMP_30_ENABLED
-        this_thr -> th.th_info.ds.ds_tid = 0;
-
-        /* set thread cache values */
-        this_thr -> th.th_team_nproc     = 1;
-        this_thr -> th.th_team_master    = this_thr;
-        this_thr -> th.th_team_serialized = 1;
-
-#if OMP_30_ENABLED
-        serial_team -> t.t_level        = serial_team -> t.t_parent -> t.t_level + 1;
-        serial_team -> t.t_active_level = serial_team -> t.t_parent -> t.t_active_level;
-#endif // OMP_30_ENABLED
-
-#if KMP_ARCH_X86 || KMP_ARCH_X86_64
-        if ( __kmp_inherit_fp_control ) {
-            __kmp_store_x87_fpu_control_word( &serial_team->t.t_x87_fpu_control_word );
-            __kmp_store_mxcsr( &serial_team->t.t_mxcsr );
-            serial_team->t.t_mxcsr &= KMP_X86_MXCSR_MASK;
-            serial_team->t.t_fp_control_saved = TRUE;
-        } else {
-            serial_team->t.t_fp_control_saved = FALSE;
-        }
-#endif /* KMP_ARCH_X86 || KMP_ARCH_X86_64 */
-        /* check if we need to allocate dispatch buffers stack */
-        KMP_DEBUG_ASSERT(serial_team->t.t_dispatch);
-        if ( !serial_team->t.t_dispatch->th_disp_buffer ) {
-            serial_team->t.t_dispatch->th_disp_buffer = (dispatch_private_info_t *)
-                __kmp_allocate( sizeof( dispatch_private_info_t ) );
-        }
-        this_thr -> th.th_dispatch = serial_team->t.t_dispatch;
-
-        KMP_MB();
-
-    } else {
-        /* this serialized team is already being used,
-         * that's fine, just add another nested level */
-        KMP_DEBUG_ASSERT( this_thr->th.th_team == serial_team );
-        KMP_DEBUG_ASSERT( serial_team -> t.t_threads );
-        KMP_DEBUG_ASSERT( serial_team -> t.t_threads[0] == this_thr );
-        ++ serial_team -> t.t_serialized;
-        this_thr -> th.th_team_serialized = serial_team -> t.t_serialized;
-
-#if OMP_30_ENABLED
-        // Nested level will be an index in the nested nthreads array
-        int level = this_thr->th.th_team->t.t_level;
-        // Thread value exists in the nested nthreads array for the next nested level
-        if ( __kmp_nested_nth.used && ( level + 1 < __kmp_nested_nth.used ) ) {
-            this_thr->th.th_current_task->td_icvs.nproc = __kmp_nested_nth.nth[ level + 1 ];
-        }
-        serial_team -> t.t_level++;
-        KF_TRACE( 10, ( "__kmpc_serialized_parallel: T#%d increasing nesting level of serial team %p to %d\n",
-                        global_tid, serial_team, serial_team -> t.t_level ) );
-#else
-        KF_TRACE( 10, ( "__kmpc_serialized_parallel: T#%d reusing team %p for nested serialized parallel region\n",
-                        global_tid, serial_team ) );
-#endif // OMP_30_ENABLED
-
-        /* allocate/push dispatch buffers stack */
-        KMP_DEBUG_ASSERT(serial_team->t.t_dispatch);
-        {
-            dispatch_private_info_t * disp_buffer = (dispatch_private_info_t *)
-                __kmp_allocate( sizeof( dispatch_private_info_t ) );
-            disp_buffer->next = serial_team->t.t_dispatch->th_disp_buffer;
-            serial_team->t.t_dispatch->th_disp_buffer = disp_buffer;
-        }
-        this_thr -> th.th_dispatch = serial_team->t.t_dispatch;
-
-        KMP_MB();
-    }
-
-    if ( __kmp_env_consistency_check )
-        __kmp_push_parallel( global_tid, NULL );
-
-// t_level is not available in 2.5 build, so check for OMP_30_ENABLED
-#if USE_ITT_BUILD && OMP_30_ENABLED
-    // Mark the start of the "parallel" region for VTune. Only use one of frame notification scheme at the moment.
-    if ( ( __itt_frame_begin_v3_ptr && __kmp_forkjoin_frames && ! __kmp_forkjoin_frames_mode ) || KMP_ITT_DEBUG )
-    {
-        __kmp_itt_region_forking( global_tid, 1 );
-    }
-    if( ( __kmp_forkjoin_frames_mode == 1 || __kmp_forkjoin_frames_mode == 3 ) && __itt_frame_submit_v3_ptr && __itt_get_timestamp_ptr )
-    {
-#if USE_ITT_NOTIFY
-        if( this_thr->th.th_team->t.t_level == 1 ) {
-            this_thr->th.th_frame_time_serialized = __itt_get_timestamp();
-        }
-#endif
-    }
-#endif /* USE_ITT_BUILD */
-
+    __kmp_serialized_parallel(loc, global_tid); /* The implementation is now in kmp_runtime.c so that it can share static functions with
+                                                 * kmp_fork_call since the tasks to be done are similar in each case.
+                                                 */
 }
 
 /*!
@@ -699,26 +467,13 @@ __kmpc_end_serialized_parallel(ident_t *loc, kmp_int32 global_tid)
     /* If necessary, pop the internal control stack values and replace the team values */
     top = serial_team -> t.t_control_stack_top;
     if ( top && top -> serial_nesting_level == serial_team -> t.t_serialized ) {
-#if OMP_30_ENABLED
-        copy_icvs(
-                  &serial_team -> t.t_threads[0] -> th.th_current_task -> td_icvs,
-                  top );
-#else
-        serial_team -> t.t_set_nproc[0]   = top -> nproc;
-        serial_team -> t.t_set_dynamic[0] = top -> dynamic;
-        serial_team -> t.t_set_nested[0]  = top -> nested;
-        serial_team -> t.t_set_blocktime[0]   = top -> blocktime;
-        serial_team -> t.t_set_bt_intervals[0] = top -> bt_intervals;
-        serial_team -> t.t_set_bt_set[0]  = top -> bt_set;
-#endif // OMP_30_ENABLED
+        copy_icvs( &serial_team -> t.t_threads[0] -> th.th_current_task -> td_icvs, top );
         serial_team -> t.t_control_stack_top = top -> next;
         __kmp_free(top);
     }
 
-#if OMP_30_ENABLED
     //if( serial_team -> t.t_serialized > 1 )
     serial_team -> t.t_level--;
-#endif // OMP_30_ENABLED
 
     /* pop dispatch buffers stack */
     KMP_DEBUG_ASSERT(serial_team->t.t_dispatch->th_disp_buffer);
@@ -754,7 +509,6 @@ __kmpc_end_serialized_parallel(ident_t *loc, kmp_int32 global_tid)
         this_thr -> th.th_dispatch       = & this_thr -> th.th_team ->
             t.t_dispatch[ serial_team -> t.t_master_tid ];
 
-#if OMP_30_ENABLED
         __kmp_pop_current_task_from_thread( this_thr );
 
         KMP_ASSERT( this_thr -> th.th_current_task -> td_flags.executing == 0 );
@@ -771,32 +525,37 @@ __kmpc_end_serialized_parallel(ident_t *loc, kmp_int32 global_tid)
             KA_TRACE( 20, ( "__kmpc_end_serialized_parallel: T#%d restoring task_team %p / team %p\n",
                             global_tid, this_thr -> th.th_task_team, this_thr -> th.th_team ) );
         }
-#endif // OMP_30_ENABLED
-
-    }
-    else {
-
-#if OMP_30_ENABLED
+    } else {
         if ( __kmp_tasking_mode != tskm_immediate_exec ) {
             KA_TRACE( 20, ( "__kmpc_end_serialized_parallel: T#%d decreasing nesting depth of serial team %p to %d\n",
                             global_tid, serial_team, serial_team -> t.t_serialized ) );
         }
-#endif // OMP_30_ENABLED
-
     }
 
-// t_level is not available in 2.5 build, so check for OMP_30_ENABLED
-#if USE_ITT_BUILD && OMP_30_ENABLED
+#if USE_ITT_BUILD
+    kmp_uint64 cur_time = 0;
+#if  USE_ITT_NOTIFY
+    if( __itt_get_timestamp_ptr ) {
+        cur_time = __itt_get_timestamp();
+    }
+#endif /* USE_ITT_NOTIFY */
+    // Report the barrier
+    if( ( __kmp_forkjoin_frames_mode == 1 || __kmp_forkjoin_frames_mode == 3 ) && __itt_frame_submit_v3_ptr ) {
+        if( this_thr->th.th_team->t.t_level == 0 ) {
+            __kmp_itt_frame_submit( global_tid, this_thr->th.th_frame_time_serialized, cur_time, 0, loc, this_thr->th.th_team_nproc, 0 );
+        }
+    }
     // Mark the end of the "parallel" region for VTune. Only use one of frame notification scheme at the moment.
     if ( ( __itt_frame_end_v3_ptr && __kmp_forkjoin_frames && ! __kmp_forkjoin_frames_mode ) || KMP_ITT_DEBUG )
     {
         this_thr->th.th_ident = loc;
         __kmp_itt_region_joined( global_tid, 1 );
     }
-    if( ( __kmp_forkjoin_frames_mode == 1 || __kmp_forkjoin_frames_mode == 3 ) && __itt_frame_submit_v3_ptr ) {
-        if( this_thr->th.th_team->t.t_level == 0 ) {
-            __kmp_itt_frame_submit( global_tid, this_thr->th.th_frame_time_serialized, __itt_timestamp_none, 0, loc );
-        }
+    if ( ( __itt_frame_submit_v3_ptr && __kmp_forkjoin_frames_mode == 3 ) || KMP_ITT_DEBUG )
+    {
+        this_thr->th.th_ident = loc;
+        // Since barrier frame for serialized region is equal to the region we use the same begin timestamp as for the barrier.
+        __kmp_itt_frame_submit( global_tid, serial_team->t.t_region_time, cur_time, 0, loc, this_thr->th.th_team_nproc, 2 );
     }
 #endif /* USE_ITT_BUILD */
 
@@ -824,42 +583,50 @@ __kmpc_flush(ident_t *loc, ...)
     /* need explicit __mf() here since use volatile instead in library */
     KMP_MB();       /* Flush all pending memory write invalidates.  */
 
-    // This is not an OMP 3.0 feature.
-    // This macro is used here just not to let the change go to 10.1.
-    // This change will go to the mainline first.
-    #if OMP_30_ENABLED
-        #if ( KMP_ARCH_X86 || KMP_ARCH_X86_64 )
-            #if KMP_MIC
-                // fence-style instructions do not exist, but lock; xaddl $0,(%rsp) can be used.
-                // We shouldn't need it, though, since the ABI rules require that
-                // * If the compiler generates NGO stores it also generates the fence
-                // * If users hand-code NGO stores they should insert the fence
-                // therefore no incomplete unordered stores should be visible.
-            #else
-                // C74404
-                // This is to address non-temporal store instructions (sfence needed).
-                // The clflush instruction is addressed either (mfence needed).
-                // Probably the non-temporal load monvtdqa instruction should also be addressed.
-                // mfence is a SSE2 instruction. Do not execute it if CPU is not SSE2.
-                if ( ! __kmp_cpuinfo.initialized ) {
-                    __kmp_query_cpuid( & __kmp_cpuinfo );
-                }; // if
-                if ( ! __kmp_cpuinfo.sse2 ) {
-                    // CPU cannot execute SSE2 instructions.
-                } else {
-                    #if KMP_COMPILER_ICC
-                    _mm_mfence();
-                    #else
-                    __sync_synchronize();
-                    #endif // KMP_COMPILER_ICC
-                }; // if
-            #endif // KMP_MIC
-        #elif KMP_ARCH_ARM
-            // Nothing yet
+    #if ( KMP_ARCH_X86 || KMP_ARCH_X86_64 )
+        #if KMP_MIC
+            // fence-style instructions do not exist, but lock; xaddl $0,(%rsp) can be used.
+            // We shouldn't need it, though, since the ABI rules require that
+            // * If the compiler generates NGO stores it also generates the fence
+            // * If users hand-code NGO stores they should insert the fence
+            // therefore no incomplete unordered stores should be visible.
         #else
-            #error Unknown or unsupported architecture
+            // C74404
+            // This is to address non-temporal store instructions (sfence needed).
+            // The clflush instruction is addressed either (mfence needed).
+            // Probably the non-temporal load monvtdqa instruction should also be addressed.
+            // mfence is a SSE2 instruction. Do not execute it if CPU is not SSE2.
+            if ( ! __kmp_cpuinfo.initialized ) {
+                __kmp_query_cpuid( & __kmp_cpuinfo );
+            }; // if
+            if ( ! __kmp_cpuinfo.sse2 ) {
+                // CPU cannot execute SSE2 instructions.
+            } else {
+                #if KMP_COMPILER_ICC || KMP_COMPILER_MSVC
+                _mm_mfence();
+                #else
+                __sync_synchronize();
+                #endif // KMP_COMPILER_ICC
+            }; // if
+        #endif // KMP_MIC
+    #elif KMP_ARCH_ARM
+        // Nothing yet
+    #elif KMP_ARCH_PPC64
+        // Nothing needed here (we have a real MB above).
+        #if KMP_OS_CNK
+        // The flushing thread needs to yield here; this prevents a
+       // busy-waiting thread from saturating the pipeline. flush is
+          // often used in loops like this:
+           // while (!flag) {
+           //   #pragma omp flush(flag)
+           // }
+       // and adding the yield here is good for at least a 10x speedup
+          // when running >2 threads per core (on the NAS LU benchmark).
+            __kmp_yield(TRUE);
         #endif
-    #endif // OMP_30_ENABLED
+    #else
+        #error Unknown or unsupported architecture
+    #endif
 
 }
 
@@ -877,6 +644,8 @@ Execute a barrier.
 void
 __kmpc_barrier(ident_t *loc, kmp_int32 global_tid)
 {
+    KMP_COUNT_BLOCK(OMP_BARRIER);
+    KMP_TIME_BLOCK(OMP_barrier);
     int explicit_barrier_flag;
     KC_TRACE( 10, ("__kmpc_barrier: called T#%d\n", global_tid ) );
 
@@ -912,6 +681,7 @@ __kmpc_barrier(ident_t *loc, kmp_int32 global_tid)
 kmp_int32
 __kmpc_master(ident_t *loc, kmp_int32 global_tid)
 {
+    KMP_COUNT_BLOCK(OMP_MASTER);
     int status = 0;
 
     KC_TRACE( 10, ("__kmpc_master: called T#%d\n", global_tid ) );
@@ -1020,11 +790,6 @@ __kmpc_end_ordered( ident_t * loc, kmp_int32 gtid )
         __kmp_parallel_dxo( & gtid, & cid, loc );
 }
 
-inline void
-__kmp_static_yield( int arg ) { // AC: needed in macro __kmp_acquire_user_lock_with_checks
-    __kmp_yield( arg );
-}
-
 static kmp_user_lock_p
 __kmp_get_critical_section_ptr( kmp_critical_name * crit, ident_t const * loc, kmp_int32 gtid )
 {
@@ -1088,6 +853,7 @@ This function blocks until the executing thread can enter the critical section.
 */
 void
 __kmpc_critical( ident_t * loc, kmp_int32 global_tid, kmp_critical_name * crit ) {
+    KMP_COUNT_BLOCK(OMP_CRITICAL);
 
     kmp_user_lock_p lck;
 
@@ -1200,6 +966,9 @@ __kmpc_barrier_master(ident_t *loc, kmp_int32 global_tid)
     if ( __kmp_env_consistency_check )
         __kmp_check_barrier( global_tid, ct_barrier, loc );
 
+#if USE_ITT_NOTIFY
+    __kmp_threads[global_tid]->th.th_ident = loc;
+#endif
     status = __kmp_barrier( bs_plain_barrier, global_tid, TRUE, 0, NULL, NULL );
 
     return (status != 0) ? 0 : 1;
@@ -1249,6 +1018,9 @@ __kmpc_barrier_master_nowait( ident_t * loc, kmp_int32 global_tid )
         __kmp_check_barrier( global_tid, ct_barrier, loc );
     }
 
+#if USE_ITT_NOTIFY
+    __kmp_threads[global_tid]->th.th_ident = loc;
+#endif
     __kmp_barrier( bs_plain_barrier, global_tid, FALSE, 0, NULL, NULL );
 
     ret = __kmpc_master (loc, global_tid);
@@ -1286,6 +1058,7 @@ introduce an explicit barrier if it is required.
 kmp_int32
 __kmpc_single(ident_t *loc, kmp_int32 global_tid)
 {
+    KMP_COUNT_BLOCK(OMP_SINGLE);
     kmp_int32 rc = __kmp_enter_single( global_tid, loc, TRUE );
     return rc;
 }
@@ -1359,8 +1132,6 @@ ompc_set_nested( int flag )
     set__nested( thread, flag ? TRUE : FALSE );
 }
 
-#if OMP_30_ENABLED
-
 void
 ompc_set_max_active_levels( int max_active_levels )
 {
@@ -1390,8 +1161,6 @@ ompc_get_team_size( int level )
     return __kmp_get_team_size( __kmp_entry_gtid(), level );
 }
 
-#endif // OMP_30_ENABLED
-
 void
 kmpc_set_stacksize( int arg )
 {
@@ -1433,12 +1202,10 @@ kmpc_set_defaults( char const * str )
     __kmp_aux_set_defaults( str, strlen( str ) );
 }
 
-#ifdef OMP_30_ENABLED
-
 int
 kmpc_set_affinity_mask_proc( int proc, void **mask )
 {
-#if defined(KMP_STUB) || !(KMP_OS_WINDOWS || KMP_OS_LINUX)
+#if defined(KMP_STUB) || !KMP_AFFINITY_SUPPORTED
     return -1;
 #else
     if ( ! TCR_4(__kmp_init_middle) ) {
@@ -1451,7 +1218,7 @@ kmpc_set_affinity_mask_proc( int proc, void **mask )
 int
 kmpc_unset_affinity_mask_proc( int proc, void **mask )
 {
-#if defined(KMP_STUB) || !(KMP_OS_WINDOWS || KMP_OS_LINUX)
+#if defined(KMP_STUB) || !KMP_AFFINITY_SUPPORTED
     return -1;
 #else
     if ( ! TCR_4(__kmp_init_middle) ) {
@@ -1464,7 +1231,7 @@ kmpc_unset_affinity_mask_proc( int proc, void **mask )
 int
 kmpc_get_affinity_mask_proc( int proc, void **mask )
 {
-#if defined(KMP_STUB) || !(KMP_OS_WINDOWS || KMP_OS_LINUX)
+#if defined(KMP_STUB) || !KMP_AFFINITY_SUPPORTED
     return -1;
 #else
     if ( ! TCR_4(__kmp_init_middle) ) {
@@ -1474,7 +1241,6 @@ kmpc_get_affinity_mask_proc( int proc, void **mask )
 #endif
 }
 
-#endif /* OMP_30_ENABLED */
 
 /* -------------------------------------------------------------------------- */
 /*!
@@ -1539,6 +1305,9 @@ __kmpc_copyprivate( ident_t *loc, kmp_int32 gtid, size_t cpy_size, void *cpy_dat
     if (didit) *data_ptr = cpy_data;
 
     /* This barrier is not a barrier region boundary */
+#if USE_ITT_NOTIFY
+    __kmp_threads[gtid]->th.th_ident = loc;
+#endif
     __kmp_barrier( bs_plain_barrier, gtid, FALSE , 0, NULL, NULL );
 
     if (! didit) (*cpy_func)( cpy_data, *data_ptr );
@@ -1546,6 +1315,9 @@ __kmpc_copyprivate( ident_t *loc, kmp_int32 gtid, size_t cpy_size, void *cpy_dat
     /* Consider next barrier the user-visible barrier for barrier region boundaries */
     /* Nesting checks are already handled by the single construct checks */
 
+#if USE_ITT_NOTIFY
+    __kmp_threads[gtid]->th.th_ident = loc; // TODO: check if it is needed (e.g. tasks can overwrite the location)
+#endif
     __kmp_barrier( bs_plain_barrier, gtid, FALSE , 0, NULL, NULL );
 }
 
@@ -1728,6 +1500,7 @@ __kmpc_destroy_nest_lock( ident_t * loc, kmp_int32 gtid, void ** user_lock ) {
 
 void
 __kmpc_set_lock( ident_t * loc, kmp_int32 gtid, void ** user_lock ) {
+    KMP_COUNT_BLOCK(OMP_set_lock);
     kmp_user_lock_p lck;
 
     if ( ( __kmp_user_lock_kind == lk_tas )
@@ -1872,6 +1645,8 @@ __kmpc_unset_nest_lock( ident_t *loc, kmp_int32 gtid, void **user_lock )
 int
 __kmpc_test_lock( ident_t *loc, kmp_int32 gtid, void **user_lock )
 {
+    KMP_COUNT_BLOCK(OMP_test_lock);
+    KMP_TIME_BLOCK(OMP_test_lock);
     kmp_user_lock_p lck;
     int          rc;
 
@@ -2034,9 +1809,14 @@ __kmpc_reduce_nowait(
     kmp_int32 num_vars, size_t reduce_size, void *reduce_data, void (*reduce_func)(void *lhs_data, void *rhs_data),
     kmp_critical_name *lck ) {
 
+    KMP_COUNT_BLOCK(REDUCE_nowait);
     int retval;
     PACKED_REDUCTION_METHOD_T packed_reduction_method;
-
+#if OMP_40_ENABLED
+    kmp_team_t *team;
+    kmp_info_t *th;
+    int teams_swapped = 0, task_state;
+#endif
     KA_TRACE( 10, ( "__kmpc_reduce_nowait() enter: called T#%d\n", global_tid ) );
 
     // why do we need this initialization here at all?
@@ -2051,7 +1831,25 @@ __kmpc_reduce_nowait(
     if ( __kmp_env_consistency_check )
         __kmp_push_sync( global_tid, ct_reduce, loc, NULL );
 
-    // it's better to check an assertion ASSERT( thr_state == THR_WORK_STATE )
+#if OMP_40_ENABLED
+    th = __kmp_thread_from_gtid(global_tid);
+    if( th->th.th_teams_microtask ) {   // AC: check if we are inside the teams construct?
+        team = th->th.th_team;
+        if( team->t.t_level == th->th.th_teams_level ) {
+            // this is reduction at teams construct
+            KMP_DEBUG_ASSERT(!th->th.th_info.ds.ds_tid);  // AC: check that tid == 0
+            // Let's swap teams temporarily for the reduction barrier
+            teams_swapped = 1;
+            th->th.th_info.ds.ds_tid = team->t.t_master_tid;
+            th->th.th_team = team->t.t_parent;
+            th->th.th_task_team = th->th.th_team->t.t_task_team;
+            th->th.th_team_nproc = th->th.th_team->t.t_nproc;
+            task_state = th->th.th_task_state;
+            if( th->th.th_task_team )
+                th->th.th_task_state = th->th.th_task_team->tt.tt_state;
+        }
+    }
+#endif // OMP_40_ENABLED
 
     // packed_reduction_method value will be reused by __kmp_end_reduce* function, the value should be kept in a variable
     // the variable should be either a construct-specific or thread-specific property, not a team specific property
@@ -2097,6 +1895,9 @@ __kmpc_reduce_nowait(
 
         // this barrier should be invisible to a customer and to the thread profiler
         //              (it's neither a terminating barrier nor customer's code, it's used for an internal purpose)
+#if USE_ITT_NOTIFY
+        __kmp_threads[global_tid]->th.th_ident = loc;
+#endif
         retval = __kmp_barrier( UNPACK_REDUCTION_BARRIER( packed_reduction_method ), global_tid, FALSE, reduce_size, reduce_data, reduce_func );
         retval = ( retval != 0 ) ? ( 0 ) : ( 1 );
 
@@ -2114,7 +1915,16 @@ __kmpc_reduce_nowait(
         KMP_ASSERT( 0 ); // "unexpected method"
 
     }
-
+#if OMP_40_ENABLED
+    if( teams_swapped ) {
+        // Restore thread structure
+        th->th.th_info.ds.ds_tid = 0;
+        th->th.th_team = team;
+        th->th.th_task_team = team->t.t_task_team;
+        th->th.th_team_nproc = team->t.t_nproc;
+        th->th.th_task_state = task_state;
+    }
+#endif
     KA_TRACE( 10, ( "__kmpc_reduce_nowait() exit: called T#%d: method %08x, returns %08x\n", global_tid, packed_reduction_method, retval ) );
 
     return retval;
@@ -2193,6 +2003,7 @@ __kmpc_reduce(
     void (*reduce_func)(void *lhs_data, void *rhs_data),
     kmp_critical_name *lck )
 {
+    KMP_COUNT_BLOCK(REDUCE_wait);
     int retval;
     PACKED_REDUCTION_METHOD_T packed_reduction_method;
 
@@ -2210,8 +2021,6 @@ __kmpc_reduce(
     if ( __kmp_env_consistency_check )
         __kmp_push_sync( global_tid, ct_reduce, loc, NULL );
 
-    // it's better to check an assertion ASSERT( thr_state == THR_WORK_STATE )
-
     packed_reduction_method = __kmp_determine_reduction_method( loc, global_tid, num_vars, reduce_size, reduce_data, reduce_func, lck );
     __KMP_SET_REDUCTION_METHOD( global_tid, packed_reduction_method );
 
@@ -2234,6 +2043,9 @@ __kmpc_reduce(
         //case tree_reduce_block:
         // this barrier should be visible to a customer and to the thread profiler
         //              (it's a terminating barrier on constructs if NOWAIT not specified)
+#if USE_ITT_NOTIFY
+        __kmp_threads[global_tid]->th.th_ident = loc; // needed for correct notification of frames
+#endif
         retval = __kmp_barrier( UNPACK_REDUCTION_BARRIER( packed_reduction_method ), global_tid, TRUE, reduce_size, reduce_data, reduce_func );
         retval = ( retval != 0 ) ? ( 0 ) : ( 1 );
 
@@ -2283,6 +2095,9 @@ __kmpc_end_reduce( ident_t *loc, kmp_int32 global_tid, kmp_critical_name *lck )
         __kmp_end_critical_section_reduce_block( loc, global_tid, lck );
 
         // TODO: implicit barrier: should be exposed
+#if USE_ITT_NOTIFY
+        __kmp_threads[global_tid]->th.th_ident = loc;
+#endif
         __kmp_barrier( bs_plain_barrier, global_tid, FALSE, 0, NULL, NULL );
 
     } else if( packed_reduction_method == empty_reduce_block ) {
@@ -2290,11 +2105,17 @@ __kmpc_end_reduce( ident_t *loc, kmp_int32 global_tid, kmp_critical_name *lck )
         // usage: if team size == 1, no synchronization is required ( Intel platforms only )
 
         // TODO: implicit barrier: should be exposed
+#if USE_ITT_NOTIFY
+        __kmp_threads[global_tid]->th.th_ident = loc;
+#endif
         __kmp_barrier( bs_plain_barrier, global_tid, FALSE, 0, NULL, NULL );
 
     } else if( packed_reduction_method == atomic_reduce_block ) {
 
         // TODO: implicit barrier: should be exposed
+#if USE_ITT_NOTIFY
+        __kmp_threads[global_tid]->th.th_ident = loc;
+#endif
         __kmp_barrier( bs_plain_barrier, global_tid, FALSE, 0, NULL, NULL );
 
     } else if( TEST_REDUCTION_METHOD( packed_reduction_method, tree_reduce_block ) ) {
@@ -2325,23 +2146,15 @@ __kmpc_end_reduce( ident_t *loc, kmp_int32 global_tid, kmp_critical_name *lck )
 kmp_uint64
 __kmpc_get_taskid() {
 
-    #if OMP_30_ENABLED
-
-        kmp_int32    gtid;
-        kmp_info_t * thread;
-
-        gtid = __kmp_get_gtid();
-        if ( gtid < 0 ) {
-            return 0;
-        }; // if
-        thread = __kmp_thread_from_gtid( gtid );
-        return thread->th.th_current_task->td_task_id;
-
-    #else
+    kmp_int32    gtid;
+    kmp_info_t * thread;
 
+    gtid = __kmp_get_gtid();
+    if ( gtid < 0 ) {
         return 0;
-
-    #endif
+    }; // if
+    thread = __kmp_thread_from_gtid( gtid );
+    return thread->th.th_current_task->td_task_id;
 
 } // __kmpc_get_taskid
 
@@ -2349,25 +2162,17 @@ __kmpc_get_taskid() {
 kmp_uint64
 __kmpc_get_parent_taskid() {
 
-    #if OMP_30_ENABLED
-
-        kmp_int32        gtid;
-        kmp_info_t *     thread;
-        kmp_taskdata_t * parent_task;
-
-        gtid = __kmp_get_gtid();
-        if ( gtid < 0 ) {
-            return 0;
-        }; // if
-        thread      = __kmp_thread_from_gtid( gtid );
-        parent_task = thread->th.th_current_task->td_parent;
-        return ( parent_task == NULL ? 0 : parent_task->td_task_id );
-
-    #else
+    kmp_int32        gtid;
+    kmp_info_t *     thread;
+    kmp_taskdata_t * parent_task;
 
+    gtid = __kmp_get_gtid();
+    if ( gtid < 0 ) {
         return 0;
-
-    #endif
+    }; // if
+    thread      = __kmp_thread_from_gtid( gtid );
+    parent_task = thread->th.th_current_task->td_parent;
+    return ( parent_task == NULL ? 0 : parent_task->td_task_id );
 
 } // __kmpc_get_parent_taskid
 
