@@ -1,11 +1,11 @@
 /*
  * kmp_ftn_entry.h -- Fortran entry linkage support for OpenMP.
- * $Revision: 42798 $
- * $Date: 2013-10-30 16:39:54 -0500 (Wed, 30 Oct 2013) $
+ * $Revision: 43435 $
+ * $Date: 2014-09-04 15:16:08 -0500 (Thu, 04 Sep 2014) $
  */
 
 /* <copyright>
-    Copyright (c) 1997-2013 Intel Corporation.  All Rights Reserved.
+    Copyright (c) 1997-2014 Intel Corporation.  All Rights Reserved.
 
     Redistribution and use in source and binary forms, with or without
     modification, are permitted provided that the following conditions
@@ -49,13 +49,13 @@
 #endif // __cplusplus
 
 /*
- * For compatiblity with the Gnu/MS Open MP codegen, omp_set_num_threads(),
+ * For compatibility with the Gnu/MS Open MP codegen, omp_set_num_threads(),
  * omp_set_nested(), and omp_set_dynamic() [in lowercase on MS, and w/o
  * a trailing underscore on Linux* OS] take call by value integer arguments.
  * + omp_set_max_active_levels()
  * + omp_set_schedule()
  *
- * For backward compatiblity with 9.1 and previous Intel compiler, these
+ * For backward compatibility with 9.1 and previous Intel compiler, these
  * entry points take call by reference integer arguments.
  */
 #ifdef KMP_GOMP_COMPAT
@@ -236,12 +236,10 @@ FTN_GET_LIBRARY (void)
     #endif
 }
 
-#if OMP_30_ENABLED
-
 int FTN_STDCALL
 FTN_SET_AFFINITY( void **mask )
 {
-    #if defined(KMP_STUB) || !(KMP_OS_WINDOWS || KMP_OS_LINUX)
+    #if defined(KMP_STUB) || !KMP_AFFINITY_SUPPORTED
         return -1;
     #else
         if ( ! TCR_4(__kmp_init_middle) ) {
@@ -254,7 +252,7 @@ FTN_SET_AFFINITY( void **mask )
 int FTN_STDCALL
 FTN_GET_AFFINITY( void **mask )
 {
-    #if defined(KMP_STUB) || !(KMP_OS_WINDOWS || KMP_OS_LINUX)
+    #if defined(KMP_STUB) || !KMP_AFFINITY_SUPPORTED
         return -1;
     #else
         if ( ! TCR_4(__kmp_init_middle) ) {
@@ -267,7 +265,7 @@ FTN_GET_AFFINITY( void **mask )
 int FTN_STDCALL
 FTN_GET_AFFINITY_MAX_PROC( void )
 {
-    #if defined(KMP_STUB) || !(KMP_OS_WINDOWS || KMP_OS_LINUX)
+    #if defined(KMP_STUB) || !KMP_AFFINITY_SUPPORTED
         return 0;
     #else
         //
@@ -282,7 +280,7 @@ FTN_GET_AFFINITY_MAX_PROC( void )
 
     #if KMP_OS_WINDOWS && KMP_ARCH_X86_64
         if ( __kmp_num_proc_groups <= 1 ) {
-            return KMP_CPU_SETSIZE;
+            return (int)KMP_CPU_SETSIZE;
         }
     #endif /* KMP_OS_WINDOWS && KMP_ARCH_X86_64 */
         return __kmp_xproc;
@@ -292,7 +290,7 @@ FTN_GET_AFFINITY_MAX_PROC( void )
 void FTN_STDCALL
 FTN_CREATE_AFFINITY_MASK( void **mask )
 {
-    #if defined(KMP_STUB) || !(KMP_OS_WINDOWS || KMP_OS_LINUX)
+    #if defined(KMP_STUB) || !KMP_AFFINITY_SUPPORTED
         *mask = NULL;
     #else
         //
@@ -309,7 +307,7 @@ FTN_CREATE_AFFINITY_MASK( void **mask )
 void FTN_STDCALL
 FTN_DESTROY_AFFINITY_MASK( void **mask )
 {
-    #if defined(KMP_STUB) || !(KMP_OS_WINDOWS || KMP_OS_LINUX)
+    #if defined(KMP_STUB) || !KMP_AFFINITY_SUPPORTED
         // Nothing
     #else
         //
@@ -331,7 +329,7 @@ FTN_DESTROY_AFFINITY_MASK( void **mask )
 int FTN_STDCALL
 FTN_SET_AFFINITY_MASK_PROC( int KMP_DEREF proc, void **mask )
 {
-    #if defined(KMP_STUB) || !(KMP_OS_WINDOWS || KMP_OS_LINUX)
+    #if defined(KMP_STUB) || !KMP_AFFINITY_SUPPORTED
         return -1;
     #else
         if ( ! TCR_4(__kmp_init_middle) ) {
@@ -344,7 +342,7 @@ FTN_SET_AFFINITY_MASK_PROC( int KMP_DEREF proc, void **mask )
 int FTN_STDCALL
 FTN_UNSET_AFFINITY_MASK_PROC( int KMP_DEREF proc, void **mask )
 {
-    #if defined(KMP_STUB) || !(KMP_OS_WINDOWS || KMP_OS_LINUX)
+    #if defined(KMP_STUB) || !KMP_AFFINITY_SUPPORTED
         return -1;
     #else
         if ( ! TCR_4(__kmp_init_middle) ) {
@@ -357,7 +355,7 @@ FTN_UNSET_AFFINITY_MASK_PROC( int KMP_DEREF proc, void **mask )
 int FTN_STDCALL
 FTN_GET_AFFINITY_MASK_PROC( int KMP_DEREF proc, void **mask )
 {
-    #if defined(KMP_STUB) || !(KMP_OS_WINDOWS || KMP_OS_LINUX)
+    #if defined(KMP_STUB) || !KMP_AFFINITY_SUPPORTED
         return -1;
     #else
         if ( ! TCR_4(__kmp_init_middle) ) {
@@ -367,8 +365,6 @@ FTN_GET_AFFINITY_MASK_PROC( int KMP_DEREF proc, void **mask )
     #endif
 }
 
-#endif /* OMP_30_ENABLED */
-
 
 /* ------------------------------------------------------------------------ */
 
@@ -410,12 +406,8 @@ xexpand(FTN_GET_MAX_THREADS)( void )
         }
         gtid   = __kmp_entry_gtid();
         thread = __kmp_threads[ gtid ];
-        #if OMP_30_ENABLED
         //return thread -> th.th_team -> t.t_current_task[ thread->th.th_info.ds.ds_tid ] -> icvs.nproc;
 	return thread -> th.th_current_task -> td_icvs.nproc;
-        #else
-        return thread -> th.th_team -> t.t_set_nproc[ thread->th.th_info.ds.ds_tid ];
-        #endif
     #endif
 }
 
@@ -427,11 +419,11 @@ xexpand(FTN_GET_THREAD_NUM)( void )
     #else
         int gtid;
 
-        #if KMP_OS_DARWIN
+        #if KMP_OS_DARWIN || KMP_OS_FREEBSD
             gtid = __kmp_entry_gtid();
         #elif KMP_OS_WINDOWS
             if (!__kmp_init_parallel ||
-                (gtid = ((kmp_intptr_t)TlsGetValue( __kmp_gtid_threadprivate_key ))) == 0) {
+                (gtid = (int)((kmp_intptr_t)TlsGetValue( __kmp_gtid_threadprivate_key ))) == 0) {
                 // Either library isn't initialized or thread is not registered
                 // 0 is the correct TID in this case
                 return 0;
@@ -482,7 +474,6 @@ xexpand(FTN_GET_NUM_PROCS)( void )
     #ifdef KMP_STUB
         return 1;
     #else
-        int gtid;
         if ( ! TCR_4(__kmp_init_middle) ) {
             __kmp_middle_initialize();
         }
@@ -553,7 +544,7 @@ xexpand(FTN_IN_PARALLEL)( void )
     #else
         kmp_info_t *th = __kmp_entry_thread();
 #if OMP_40_ENABLED
-        if ( th->th.th_team_microtask ) {
+        if ( th->th.th_teams_microtask ) {
             // AC: r_in_parallel does not work inside teams construct
             //     where real parallel is inactive, but all threads have same root,
             //     so setting it in one team affects other teams.
@@ -566,8 +557,6 @@ xexpand(FTN_IN_PARALLEL)( void )
     #endif
 }
 
-#if OMP_30_ENABLED
-
 void FTN_STDCALL
 xexpand(FTN_SET_SCHEDULE)( kmp_sched_t KMP_DEREF kind, int KMP_DEREF modifier )
 {
@@ -687,8 +676,6 @@ xexpand(FTN_IN_FINAL)( void )
     #endif
 }
 
-#endif // OMP_30_ENABLED
-
 #if OMP_40_ENABLED
 
 
@@ -709,7 +696,7 @@ xexpand(FTN_GET_NUM_TEAMS)( void )
         return 1;
     #else
         kmp_info_t *thr = __kmp_entry_thread();
-        if ( thr->th.th_team_microtask ) {
+        if ( thr->th.th_teams_microtask ) {
             kmp_team_t *team = thr->th.th_team;
             int tlevel = thr->th.th_teams_level;
             int ii = team->t.t_level;            // the level of the teams construct
@@ -748,7 +735,7 @@ xexpand(FTN_GET_TEAM_NUM)( void )
         return 0;
     #else
         kmp_info_t *thr = __kmp_entry_thread();
-        if ( thr->th.th_team_microtask ) {
+        if ( thr->th.th_teams_microtask ) {
             kmp_team_t *team = thr->th.th_team;
             int tlevel = thr->th.th_teams_level; // the level of the teams construct
             int ii = team->t.t_level;
@@ -1032,7 +1019,7 @@ FTN_SET_DEFAULTS( char const * str
 {
     #ifndef KMP_STUB
         #ifdef PASS_ARGS_BY_VALUE
-            int len = strlen( str );
+            int len = (int)strlen( str );
         #endif
         __kmp_aux_set_defaults( str, len );
     #endif
@@ -1068,19 +1055,19 @@ FTN_GET_CANCELLATION_STATUS(int cancel_kind) {
 #endif // OMP_40_ENABLED
 
 // GCC compatibility (versioned symbols)
-#if KMP_OS_LINUX
+#ifdef KMP_USE_VERSION_SYMBOLS
 
 /*
     These following sections create function aliases (dummy symbols) for the omp_* routines.
-    These aliases will then be versioned according to how libgomp ``versions'' its 
-    symbols (OMP_1.0, OMP_2.0, OMP_3.0, ...) while also retaining the 
+    These aliases will then be versioned according to how libgomp ``versions'' its
+    symbols (OMP_1.0, OMP_2.0, OMP_3.0, ...) while also retaining the
     default version which libiomp5 uses: VERSION (defined in exports_so.txt)
-    If you want to see the versioned symbols for libgomp.so.1 then just type: 
+    If you want to see the versioned symbols for libgomp.so.1 then just type:
 
     objdump -T /path/to/libgomp.so.1 | grep omp_
 
     Example:
-    Step 1)  Create __kmp_api_omp_set_num_threads_10_alias 
+    Step 1)  Create __kmp_api_omp_set_num_threads_10_alias
              which is alias of __kmp_api_omp_set_num_threads
     Step 2)  Set __kmp_api_omp_set_num_threads_10_alias to version: omp_set_num_threads@OMP_1.0
     Step 2B) Set __kmp_api_omp_set_num_threads to default version : omp_set_num_threads@@VERSION
@@ -1112,7 +1099,6 @@ xaliasify(FTN_TEST_NEST_LOCK,    10);
 xaliasify(FTN_GET_WTICK, 20);
 xaliasify(FTN_GET_WTIME, 20);
 
-#if OMP_30_ENABLED
 // OMP_3.0 aliases
 xaliasify(FTN_SET_SCHEDULE,            30);
 xaliasify(FTN_GET_SCHEDULE,            30);
@@ -1136,7 +1122,6 @@ xaliasify(FTN_TEST_NEST_LOCK,          30);
 
 // OMP_3.1 aliases
 xaliasify(FTN_IN_FINAL, 31);
-#endif /* OMP_30_ENABLED */
 
 #if OMP_40_ENABLED
 // OMP_4.0 aliases
@@ -1180,7 +1165,6 @@ xversionify(FTN_TEST_NEST_LOCK,    10, "OMP_1.0");
 xversionify(FTN_GET_WTICK,         20, "OMP_2.0");
 xversionify(FTN_GET_WTIME,         20, "OMP_2.0");
 
-#if OMP_30_ENABLED
 // OMP_3.0 versioned symbols
 xversionify(FTN_SET_SCHEDULE,      30, "OMP_3.0");
 xversionify(FTN_GET_SCHEDULE,      30, "OMP_3.0");
@@ -1206,7 +1190,6 @@ xversionify(FTN_TEST_NEST_LOCK,    30, "OMP_3.0");
 
 // OMP_3.1 versioned symbol
 xversionify(FTN_IN_FINAL,          31, "OMP_3.1");
-#endif /* OMP_30_ENABLED */
 
 #if OMP_40_ENABLED
 // OMP_4.0 versioned symbols
@@ -1224,7 +1207,7 @@ xversionify(FTN_GET_CANCELLATION,  40, "OMP_4.0");
 // OMP_5.0 versioned symbols
 #endif
 
-#endif /* KMP_OS_LINUX */
+#endif // KMP_USE_VERSION_SYMBOLS
 
 #ifdef __cplusplus
     } //extern "C"
