@@ -1,11 +1,11 @@
 /*
  * kmp_i18n.c
- * $Revision: 42810 $
- * $Date: 2013-11-07 12:06:33 -0600 (Thu, 07 Nov 2013) $
+ * $Revision: 43084 $
+ * $Date: 2014-04-15 09:15:14 -0500 (Tue, 15 Apr 2014) $
  */
 
 /* <copyright>
-    Copyright (c) 2007-2013 Intel Corporation.  All Rights Reserved.
+    Copyright (c) 2007-2014 Intel Corporation.  All Rights Reserved.
 
     Redistribution and use in source and binary forms, with or without
     modification, are permitted provided that the following conditions
@@ -380,7 +380,7 @@ __kmp_i18n_do_catopen(
                 reinterpret_cast< LPCSTR >( & __kmp_i18n_do_catopen ),
                 & handle
             );
-        if ( ! brc ) {    // Error occured.
+        if ( ! brc ) {    // Error occurred.
             status = KMP_I18N_ABSENT;  // mark catalog as absent so it will not be re-opened.
             goto end;
             // TODO: Enable multiple messages (KMP_MSG) to be passed to __kmp_msg; and print
@@ -390,7 +390,7 @@ __kmp_i18n_do_catopen(
         // Now get path to the our DLL.
         for ( ; ; ) {
             DWORD drc = GetModuleFileName( handle, path.str, path.size );
-            if ( drc == 0 ) {    // Error occured.
+            if ( drc == 0 ) {    // Error occurred.
                 status = KMP_I18N_ABSENT;
                 goto end;
             }; // if
@@ -492,7 +492,7 @@ __kmp_i18n_catclose(
 /*
     We use FormatMessage() to get strings from catalog, get system error messages, etc.
     FormatMessage() tends to return Windows* OS-style end-of-lines, "\r\n". When string is printed,
-    printf() also replaces all the occurences of "\n" with "\r\n" (again!), so sequences like
+    printf() also replaces all the occurrences of "\n" with "\r\n" (again!), so sequences like
     "\r\r\r\n" appear in output. It is not too good.
 
     Additional mess comes from message catalog: Our catalog source en_US.mc file (generated by
@@ -511,7 +511,7 @@ __kmp_i18n_catclose(
     Thus, stripping all "\r" normalizes string and returns it to canonical form, so printf() will
     produce correct end-of-line sequences.
 
-    ___strip_crs() serves for this purpose: it removes all the occurences of "\r" in-place and
+    ___strip_crs() serves for this purpose: it removes all the occurrences of "\r" in-place and
     returns new length of string.
 */
 static
@@ -834,7 +834,7 @@ sys_error(
                 // not issue warning if strerror_r() returns `int' instead of expected `char *'.
             message = __kmp_str_format( "%s", err_msg );
 
-        #else // OS X*
+        #else // OS X*, FreeBSD* etc.
 
             // XSI version of strerror_r.
 
