@@ -1,11 +1,11 @@
 /*
  * kmp_taskq.c -- TASKQ support for OpenMP.
- * $Revision: 42582 $
- * $Date: 2013-08-09 06:30:22 -0500 (Fri, 09 Aug 2013) $
+ * $Revision: 43389 $
+ * $Date: 2014-08-11 10:54:01 -0500 (Mon, 11 Aug 2014) $
  */
 
 /* <copyright>
-    Copyright (c) 1997-2013 Intel Corporation.  All Rights Reserved.
+    Copyright (c) 1997-2014 Intel Corporation.  All Rights Reserved.
 
     Redistribution and use in source and binary forms, with or without
     modification, are permitted provided that the following conditions
@@ -52,23 +52,6 @@
 
 #define THREAD_ALLOC_FOR_TASKQ
 
-static void
-__kmp_static_delay( int arg )
-{
-/* Work around weird code-gen bug that causes assert to trip */
-#if KMP_ARCH_X86_64 && KMP_OS_LINUX
-    KMP_ASSERT( arg != 0 );
-#else
-    KMP_ASSERT( arg >= 0 );
-#endif
-}
-
-static void
-__kmp_static_yield( int arg )
-{
-    __kmp_yield( arg );
-}
-
 static int
 in_parallel_context( kmp_team_t *team )
 {
@@ -809,7 +792,7 @@ __kmp_dequeue_task (kmp_int32 global_tid, kmpc_task_queue_t *queue, int in_paral
  * 1.  Walk up the task queue tree from the current queue's parent and look
  *      on the way up (for loop, below).
  * 2.  Do a depth-first search back down the tree from the root and
- *      look (find_task_in_descandent_queue()).
+ *      look (find_task_in_descendant_queue()).
  *
  * Here are the rules for deciding which task to take from a queue
  * (__kmp_find_task_in_queue ()):
@@ -1627,7 +1610,6 @@ __kmpc_end_taskq(ident_t *loc, kmp_int32 global_tid, kmpc_thunk_t *taskq_thunk)
                  && (! __kmp_taskq_has_any_children(queue) )
                  && (! (queue->tq_flags & TQF_ALL_TASKS_QUEUED) )
                   ) {
-                __kmp_static_delay( 1 );
                 KMP_YIELD_WHEN( TRUE, spins );
             }
 
